#include "GraphicsEngine.h"
#include "Parser.h"
#include "AllObjects.h"
#include "Functions.h"

#include <vector>

float unitSize = 100;

void start()
{
	// Make the parser and the graphics engine
	Parser parser(unitSize);
	GraphicsEngine graphicsEngine;

	vector<GameObject*>* objects;

	int roomNo = 1;
	bool bContinue = true;

	while (bContinue)
	{ 
		if (!parser.parseRoom("../../level" + Functions::toString(roomNo) + ".dat"))
		{
			bContinue = false;
			continue;
		}
		objects = parser.getObjects();
		
		// Create the room itself (planes, objects etc)
		graphicsEngine.reset();
		graphicsEngine.createPlane("ground", Vec3(parser.getRoomWidth()/2,0,parser.getRoomDepth()/2), parser.getRoomWidth()*2,parser.getRoomDepth()*2, 0, Vec3(0,1,0), parser.getGroundTexture());
		graphicsEngine.createSky(max(parser.getRoomWidth(), parser.getRoomDepth()), parser.getSkyTexture());
		for (unsigned int i = 0; i < objects->size(); i++)
		{
			Wall* wall = dynamic_cast<Wall*>(objects->at(i));
			if (wall != NULL)
				graphicsEngine.createWall(wall, parser.getWallTexture());

			GameBox* box = dynamic_cast<GameBox*>(objects->at(i));
			if (box != NULL)
				graphicsEngine.createBox(box, parser.getBoxTexture());

			Player* player = dynamic_cast<Player*>(objects->at(i));
			if (player != NULL)
				graphicsEngine.createPlayer(player);

			Goal* goal = dynamic_cast<Goal*>(objects->at(i));
			if (goal != NULL)
				graphicsEngine.createGoal(goal);
		}		

		// Start rendering. This methods ends when the player wants to quit (returns false) or when he goes
		// to the next room (returns true)
		bContinue = graphicsEngine.startRenderLoop();

		// Clear up
		objects->clear();

		roomNo++;
	}
	delete objects;
}


#if OGRE_PLATFORM == PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WIN32
#define WIN32_LEAN_AND_MEAN
#include "windows.h"

INT WINAPI WinMain(HINSTANCE hInst, HINSTANCE, LPSTR strCmdLine, INT)
#else
int main(int argc, char **argv)
#endif
{
    try
    {
		start();
    }
    catch(Exception& e)
    {
#if OGRE_PLATFORM == PLATFORM_WIN32 || OGRE_PLATFORM == OGRE_PLATFORM_WIN32
        MessageBoxA(NULL, e.getFullDescription().c_str(), "An exception has occurred!", MB_OK | MB_ICONERROR | MB_TASKMODAL);
#else
        fprintf(stderr, "An exception has occurred: %s\n",
            e.getFullDescription().c_str());
#endif
    }

    return 0;
}