#include "GameBox.h"
#include <cmath>


GameBox::GameBox(float x, float y, float z, float width, float height, float depth, bool bStatic, string name) : Cube(x,y,z, width, height, depth, bStatic, name)
{
	newPos = Vec3(x,y,z);
}

GameBox::~GameBox()
{
}

void GameBox::setNewPos(float x, float y, float z)
{
	if (pos == newPos)
		newPos = Vec3(x,y,z);
}

Vec3 GameBox::toStartPos()
{	
	newPos.set(startPos);
	pos.set(startPos);
	return pos;
}

bool GameBox::isMoving()
{
	return pos != newPos;
}
/**
 *	Moves to the new position. The paramater is used to keep a steady speed
 */
Vec3 GameBox::moveToNewPos(float timeSinceLastFrame)
{
	float moveSpeed = 50*timeSinceLastFrame;
	float dx = pos.getX() - newPos.getX();
	float dz = pos.getZ() - newPos.getZ();
	if (dx == 0 && dz == 0)
		return pos;

	// move in x dir
	if (pos.getX() < newPos.getX())
	{
		pos.setX(min(pos.getX() + moveSpeed, newPos.getX()));
	}
	else
	{
		if (pos.getX() > newPos.getX())
		{
			pos.setX(max(pos.getX() - moveSpeed, newPos.getX()));
		}
	}

	// move in z dir
	if (pos.getZ() < newPos.getZ())
	{
		pos.setZ(min(pos.getZ() + moveSpeed, newPos.getZ()));
	}
	else
	{
		if (pos.getZ() > newPos.getZ())
		{
			pos.setZ(max(pos.getZ() - moveSpeed, newPos.getZ()));
		}
	}

	return pos;
}

