#ifndef GRAPHICSENGINE_H
#define GRAPHICSENGINE_H

#include <vector>
#include <Ogre.h>
#include <OIS/OIS.h>
#include <CEGUI/CEGUI.h>
#include <OgreCEGUIRenderer.h>
#include "InputListener.h"
#include "AllObjects.h"
#include "Vec3.h"

using namespace Ogre;

class GraphicsEngine
{
public:
    GraphicsEngine();
    ~GraphicsEngine();

	bool startRenderLoop();
	void reset();

	// Create object methods
	void createSky(float size, string texture);
	void createWall(Wall* wall, string texture);
	void createBox(GameBox* box, string texture);
	void createPlayer(Player* player);
	void createGoal(Goal* goal);
	void createPlane(std::string name, Vec3 position, int width, int height, int offset, Vec3 orientation, string texture);

	void createMiniMap(int x, int z);
private:
    Root *mRoot;
    CEGUI::OgreCEGUIRenderer *mRenderer;
    CEGUI::System *mSystem;
    InputListener *mListener;
	RenderWindow *mWin;

	vector<GameObject*>* mStaticObjects;
	vector<GameBox*>* mBoxes;
	Player* mPlayer;

	void createRoot();    
    void defineResources();    
    void setupRenderSystem();    
    void createRenderWindow();
    void initializeResourceGroups();
	void setupScene();
	void clearScene();
    void setupCEGUI();
    void createFrameListener();
};

#endif // GRAPHICSENGINE_H