#ifndef INPUTLISTENER_H
#define INPUTLISTENER_H

#include "AllObjects.h"

#include <vector>
#include <Ogre.h>
#include <OIS/OIS.h>

using namespace Ogre;

class InputListener : public FrameListener, public OIS::MouseListener, public OIS::KeyListener
{
public:
    InputListener(OIS::ParamList pl, SceneManager *sceneMgr);
	~InputListener();

	void setupRoom(vector<GameObject*>* staticObjects, vector<GameBox*>* boxes);
	bool getRoomEnd();
	void setPlayer(Player* player);

    bool frameStarted(const FrameEvent& evt);
   
	// MouseListener
    bool mouseMoved(const OIS::MouseEvent &e);
	bool mousePressed(const OIS::MouseEvent &e, OIS::MouseButtonID id);
	bool mouseReleased(const OIS::MouseEvent &e, OIS::MouseButtonID id);

	// KeyListener
    bool keyPressed(const OIS::KeyEvent &e);
    bool keyReleased(const OIS::KeyEvent &e);

protected:
    Real mRotate;          // The rotate constant
    Real mMove;            // The movement constant

    SceneManager *mSceneMgr;   // The current SceneManager
    SceneNode *mCamNode;   // The SceneNode the camera is currently attached to
	SceneNode *mCamYawNode;
	SceneNode *mCamPitchNode;

    bool mContinue;        // Whether to continue rendering or not
    Vector3 mDirection;     // Value to move in the correct direction
private:
	void showTopOverlay(String text);
	void showBigOverlay(String text);
	
	// Camera movement methods
	void moveCamera(Real timeSinceLastFrame);
	bool moveToPos(Vector3 pos, Vector3 oldPos);
	bool moveBox(Vector3 pos, Vector3 oldPos);
	void zoomMiniMap(int amount);
	GameObject* isPosFree(Vector3 pos);
	GameBox* isBoxFree(Vector3 pos);

	void restartRoom();
	void endRoom();

	void updateObjects(Real timeSinceLastFrame);

	// Variables
	OIS::InputManager *mInputManager;
    OIS::Keyboard *mKeyboard;
	OIS::Mouse *mMouse;

	Overlay* topOverlay;
	Overlay* bigOverlay;

	double overlayTime;
	
	// Game variables
	int noOfPushes;
	bool bRoomEnd;
	vector<GameObject*>* mStaticObjects;
	vector<GameBox*>* mBoxes;
	Player* mPlayer;
};

#endif // INPUTLISTENER_H