#include "Parser.h"
#include <iostream>
#include <fstream>
#include <string>

#include "AllObjects.h"


Parser::Parser(float unitSize)
{
	this->unitSize = unitSize;
	objects = new vector<GameObject*>;
}

Parser::~Parser(void)
{
}

/**
 *	Returns the pointer to the objects. Note that the one getting this pointer needs
 *	to delete the vector
 */
vector<GameObject*>* Parser::getObjects()
{
	return objects;
}

int Parser::getRoomWidth()
{
	return roomWidth;
}

int Parser::getRoomDepth()
{
	return roomDepth;
}

/**
 *	Parses a given file. It parses each char as a possible object for a room
 */
bool Parser::parseRoom(string filename)
{
	ifstream data(filename.c_str());
	if (data.is_open()) 
	{
		getline(data, groundTexture);
		getline(data, skyTexture);
		getline(data, wallTexture);
		getline(data, boxTexture);
		string line;
		int lineNo = 0;
		while ( !data.eof() ) 
		{
			lineNo++;

			getline(data,line);
			for (unsigned int i=0; i < line.length(); i++)
			{
				GameObject* object = parseChar(line.at(i), lineNo, i);
				if (object != NULL)
				{
					objects->push_back(object);
				}
			}
		}
		roomDepth = lineNo*100;
		roomWidth = (int) line.length()*100;

		return true;
	}
	else
	{
		cout << "Unable to open file" << endl;
		return false;
	}
}

/**
 *	Parses the given char to any of the known objects.
 *
 *	@param:	The given char
 *	@param: The number of the line the char came from. used to calculate a position
 *	@param:	The pos of the char in the line. Also used to calculate a position
 */
GameObject* Parser::parseChar(char c, int lineNo, int pos)
{
	float x = float(pos*unitSize + unitSize/2);
	float y = float(unitSize/2);
	float z = float(lineNo*unitSize - unitSize/2);

	int Doornr = 0;

	switch(c)
	{
		case '#':	return new Wall(x,y,z,unitSize,unitSize,unitSize,true);
					break;
		case '*':	return new GameBox(x,y,z,unitSize,unitSize,unitSize,true);
					break;
		case 'S':	return new Player(x,float(0.4*unitSize),z,false,"Player");
					break;
		case 'G':	return new Goal(x,float(0.4*unitSize),z,15,"Goal");
					break;
		default:	return NULL;
	}
}

string Parser::getGroundTexture()
{
	return groundTexture;
}

string Parser::getSkyTexture()
{
	return skyTexture;
}

string Parser::getWallTexture()
{
	return wallTexture;
}

string Parser::getBoxTexture()
{
	return boxTexture;
}